/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

/**
 * The nsIDOMBeforeUnloadEvent interface is the interface for events
 * sent to handlers of the "beforeunload" event. This event is
 * non-standard. Interface derived from Microsoft IE's event
 * implementation.
 *
 * http://msdn.microsoft.com/library/default.asp?url=/workshop/author/dhtml/reference/events.asp
 *
 */

[builtinclass, uuid(ee7b8943-440e-4cc3-b25e-c17c8a821d8b)]
interface nsIDOMBeforeUnloadEvent : nsIDOMEvent
{
  /**
   * Attribute used to pass back a return value from a beforeunload
   * handler
   */
           attribute DOMString          returnValue;
};
