/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
#include "nsIOutputStream.idl"

[scriptable, uuid(4DA56F10-99FE-11d2-8EBB-00805F29F370)]
interface nsIRDFXMLSource : nsISupports
{
    /**
     * Serialize the contents of the datasource to aStream.
     * @param aStream the output stream the will receive the
     *   RDF/XML. Currently, the output stream need only
     *   implement the |write()| method.
     */
    void Serialize(in nsIOutputStream aStream);
};

